from schemazer import RequestErrors
from schemazer.base import SchemazerMethod
from schemazer.commons.errors import AuthErrors
from schemazer.commons.limits import AllLimit
from schemazer.commons.parameters import AuthParameters


class TestMethodsGroup:
    _name = 'auth'
    test_method = SchemazerMethod(
        name='test_method',
        description='Test method',
        parameters=[
            AuthParameters.Email(),
        ],
        errors=[RequestErrors.BadRequest,
                AuthErrors.UserNotFound],
        limits=[AllLimit()]
    )


class TestSchema:
    test = TestMethodsGroup



def remote_addr(request):
    addr = request.environ.get('HTTP_X_FORWARDED_FOR',
                               request.remote_addr).split(',')
    return addr[0] or request.environ.get('HTTP_X_REAL_IP', None)



from schemazer.base import SchemazerParameter
from schemazer.commons.patterns import (PhoneFormat, EmailFormat,
                                        PasswordFormat, SecretKeyFormat,
                                        TokenFormat, ApiKeyFormat)
from schemazer.validator import PatternValidator


class AuthParameters:
    class Phone(SchemazerParameter):
        name = 'phone'
        description = 'Number of phone'
        type = str
        required = True
        example = '81234567890'
        validator = PatternValidator(PhoneFormat)

    class Email(SchemazerParameter):
        name = 'email'
        description = 'Email'
        type = str
        required = True
        example = 'example@gmail.ru'
        validator = PatternValidator(EmailFormat)

    class Password(SchemazerParameter):
        name = 'password'
        description = 'Password'
        type = str
        required = True
        example = 'kf7J73nsIJN'
        validator = PatternValidator(PasswordFormat)

    class SecretKey(SchemazerParameter):
        name = 'secret_key'
        description = 'Secret key parameter'
        type = str
        required = True
        example = '9845121'
        validator = PatternValidator(SecretKeyFormat)

    class Token(SchemazerParameter):
        name = 'token'
        description = 'Registration token'
        type = str
        required = True
        example = '202cb962ac59075b964b07152d234b70'
        validator = PatternValidator(TokenFormat)

    class ApiKey(SchemazerParameter):
        name = 'apikey'
        description = 'Api secret key'
        type = str
        required = True
        example = '202cb962ac59075b'
        validator = PatternValidator(ApiKeyFormat)



import unittest

from flask import Flask, request, json
from werkzeug.exceptions import HTTPException

from schemazer import Query, RequestValidator
from schemazer.commons.errors import RequestErrors
from tests import TestSchema


class TestRequestValidator(unittest.TestCase):
    def get_abort_response_data(self, abort_response):
        self.assertTrue(hasattr(abort_response, 'response'))
        return json.loads(abort_response.response.get_data())

    def setUp(self):
        self.validator = RequestValidator()
        self.schema = TestSchema()
        self.app = Flask(__name__)

    def test_missing_required_parameter(self):
        """
        Request with missing required parameter.
        """
        with self.app.test_request_context():
            request.query = Query()
            try:
                self.validator.process(
                    self.schema.test.test_method.parameters)
            except HTTPException as abort_response:
                error = self.get_abort_response_data(abort_response)
                assert error
                self.assertIn('error', error.keys())
                error = error['error']
                assert error.get('code')
                assert RequestErrors.BadRequest.code == error['code']
                assert error.get('failure_param')

    def test_wrong_format_parameter(self):
        """
        Request with wrong parameters.
        """
        with self.app.test_request_context():
            request.query = Query()
            request.query.args = {'email': 'wrong'}
            try:
                self.validator.process(
                    self.schema.test.test_method.parameters)
            except HTTPException as abort_response:
                error = self.get_abort_response_data(abort_response)
                assert error
                self.assertIn('error', error.keys())
                error = error['error']
                assert error.get('code')
                assert RequestErrors.BadRequest.code == error['code']
                assert error.get('failure_param')


if __name__ == '__main__':
    unittest.main()



import unittest

from schemazer.base import SchemazerResponseField


class TestResponseFields(unittest.TestCase):
    def test_response_field_structure(self):
        class TestResponseField(SchemazerResponseField):
            description = 'description'
            name = 'name'
            type = int

        float_field = TestResponseField()

        self.assertEqual(
            float_field.to_dict(),
            {
                TestResponseField.name:
                    {
                        'type': int.__name__,
                        'description': TestResponseField.description,
                    }
            }
        )


if __name__ == '__main__':
    unittest.main()
